package org.gcube.portlets.user.geoportaldataviewer.shared.gis;

import java.io.Serializable;

import org.gcube.application.geoportalcommon.shared.products.BaseConcessioneDV;


/**
 * The Class LayerObject.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 * Sep 9, 2021
 */
public class LayerObject implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2282478701630148774L;
	/* This is the layer/product type. E.g. Concessione */
	private String itemType;
	private LayerItem layerItem;
	private BaseConcessioneDV sourceConcessione;

	/**
	 * Instantiates a new layer object.
	 */
	public LayerObject() {
	}

	/**
	 * Instantiates a new layer object.
	 *
	 * @param itemType the item type
	 * @param layerItem the layer item
	 * @param sourceConcessione the source concessione
	 */
	public LayerObject(String itemType, LayerItem layerItem, BaseConcessioneDV sourceConcessione) {
		super();
		this.itemType = itemType;
		this.layerItem = layerItem;
		this.sourceConcessione = sourceConcessione;
	}

	/**
	 * Gets the item type.
	 *
	 * @return the item type
	 */
	public String getItemType() {
		return itemType;
	}

	/**
	 * Gets the layer item.
	 *
	 * @return the layer item
	 */
	public LayerItem getLayerItem() {
		return layerItem;
	}

	/**
	 * Gets the source concessione.
	 *
	 * @return the source concessione
	 */
	public BaseConcessioneDV getSourceConcessione() {
		return sourceConcessione;
	}

	/**
	 * Sets the item type.
	 *
	 * @param itemType the new item type
	 */
	public void setItemType(String itemType) {
		this.itemType = itemType;
	}

	/**
	 * Sets the layer item.
	 *
	 * @param layerItem the new layer item
	 */
	public void setLayerItem(LayerItem layerItem) {
		this.layerItem = layerItem;
	}

	/**
	 * Sets the source concessione.
	 *
	 * @param sourceConcessione the new source concessione
	 */
	public void setSourceConcessione(BaseConcessioneDV sourceConcessione) {
		this.sourceConcessione = sourceConcessione;
	}

	/**
	 * To string.
	 *
	 * @return the string
	 */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("LayerObject [itemType=");
		builder.append(itemType);
		builder.append(", layerItem=");
		builder.append(layerItem);
		builder.append(", sourceConcessione=");
		builder.append(sourceConcessione);
		builder.append("]");
		return builder.toString();
	}

}